@@ -15,7 +15,7 @@ import numpy as np
 
 from hypothesis import strategies as st
 from hypothesis._settings import note_deprecation
-from hypothesis.errors import InvalidArgument
+from hypothesis.errors import HypothesisException, InvalidArgument
 from hypothesis.extra._array_helpers import (
     NDIM_MAX,
     BasicIndex,
@@ -63,6 +63,9 @@ __all__ = [
 
 TIME_RESOLUTIONS = tuple("Y  M  D  h  m  s  ms  us  ns  ps  fs  as".split())
 
+# See https://github.com/HypothesisWorks/hypothesis/pull/3394 and linked discussion.
+NP_FIXED_UNICODE = tuple(int(x) for x in np.__version__.split(".")[:2]) >= (1, 19)
+
 
 @defines_strategy(force_reusable_values=True)
 def from_dtype(
@@ -121,7 +124,7 @@ def from_dtype(
         result: st.SearchStrategy[Any] = st.booleans()
     elif dtype.kind == "f":
         result = st.floats(
-            width=8 * dtype.itemsize,
+            width=min(8 * dtype.itemsize, 64),
             **compat_kw(
                 "min_value",
                 "max_value",
@@ -157,6 +160,8 @@ def from_dtype(
     elif dtype.kind == "U":
         # Encoded in UTF-32 (four bytes/codepoint) and null-terminated
         max_size = (dtype.itemsize or 0) // 4 or None
+        if NP_FIXED_UNICODE and "alphabet" not in kwargs:
+            kwargs["alphabet"] = st.characters()
         result = st.text(**compat_kw("alphabet", "min_size", max_size=max_size)).filter(
             lambda b: b[-1:] != "\0"
         )
@@ -192,17 +197,29 @@ class ArrayStrategy(st.SearchStrategy):
                 f"Could not add element={val!r} of {val.dtype!r} to array of "
                 f"{result.dtype!r} - possible mismatch of time units in dtypes?"
             ) from err
-        if self._check_elements and val != result[idx] and val == val:
+        try:
+            elem_changed = self._check_elements and val != result[idx] and val == val
+        except Exception as err:  # pragma: no cover
+            # This branch only exists to help debug weird behaviour in Numpy,
+            # such as the string problems we had a while back.
+            raise HypothesisException(
+                "Internal error when checking element=%r of %r to array of %r"
+                % (val, val.dtype, result.dtype)
+            ) from err
+        if elem_changed:
             strategy = self.fill if fill else self.element_strategy
-            if self.dtype.kind == "f":
+            if self.dtype.kind == "f":  # pragma: no cover
+                # This logic doesn't trigger in our coverage tests under Numpy 1.24+,
+                # with built-in checks for overflow, but we keep it for good error
+                # messages and compatibility with older versions of Numpy.
                 try:
                     is_subnormal = 0 < abs(val) < np.finfo(self.dtype).tiny
-                except Exception:  # pragma: no cover
+                except Exception:
                     # val may be a non-float that does not support the
                     # operations __lt__ and __abs__
-                    is_subnormal = False  # pragma: no cover
+                    is_subnormal = False
                 if is_subnormal:
-                    raise InvalidArgument(  # pragma: no cover
+                    raise InvalidArgument(
                         f"Generated subnormal float {val} from strategy "
                         f"{strategy} resulted in {result[idx]!r}, probably "
                         "as a result of NumPy being built with flush-to-zero "
@@ -370,7 +387,7 @@ def arrays(
       strategy that generates such values.
     * ``elements`` is a strategy for generating values to put in the array.
       If it is None a suitable value will be inferred based on the dtype,
-      which may give any legal value (including eg ``NaN`` for floats).
+      which may give any legal value (including eg NaN for floats).
       If a mapping, it will be passed as ``**kwargs`` to ``from_dtype()``
     * ``fill`` is a strategy that may be used to generate a single background
       value for the array. If None, a suitable default will be inferred
@@ -382,8 +399,9 @@ def arrays(
       distinct from one another. Note that in this case multiple NaN values
       may still be allowed. If fill is also set, the only valid values for
       it to return are NaN values (anything for which :obj:`numpy:numpy.isnan`
-      returns True. So e.g. for complex numbers (nan+1j) is also a valid fill).
-      Note that if unique is set to True the generated values must be hashable.
+      returns True. So e.g. for complex numbers ``nan+1j`` is also a valid fill).
+      Note that if ``unique`` is set to ``True`` the generated values must be
+      hashable.
 
     Arrays of specified ``dtype`` and ``shape`` are generated for example
     like this:
@@ -391,17 +409,11 @@ def arrays(
     .. code-block:: pycon
 
       >>> import numpy as np
+      >>> from hypothesis import strategies as st
       >>> arrays(np.int8, (2, 3)).example()
       array([[-8,  6,  3],
              [-6,  4,  6]], dtype=int8)
-
-    - See :doc:`What you can generate and how <data>`.
-
-    .. code-block:: pycon
-
-      >>> import numpy as np
-      >>> from hypothesis.strategies import floats
-      >>> arrays(np.float, 3, elements=floats(0, 1)).example()
+      >>> arrays(np.float, 3, elements=st.floats(0, 1)).example()
       array([ 0.88974794,  0.77387938,  0.1977879 ])
 
     Array values are generated in two parts:
@@ -409,14 +421,14 @@ def arrays(
     1. Some subset of the coordinates of the array are populated with a value
        drawn from the elements strategy (or its inferred form).
     2. If any coordinates were not assigned in the previous step, a single
-       value is drawn from the fill strategy and is assigned to all remaining
+       value is drawn from the ``fill`` strategy and is assigned to all remaining
        places.
 
-    You can set fill to :func:`~hypothesis.strategies.nothing` if you want to
+    You can set :func:`fill=nothing() <hypothesis.strategies.nothing>` to
     disable this behaviour and draw a value for every element.
 
-    If fill is set to None then it will attempt to infer the correct behaviour
-    automatically: If unique is True, no filling will occur by default.
+    If ``fill=None``, then it will attempt to infer the correct behaviour
+    automatically. If ``unique`` is ``True``, no filling will occur by default.
     Otherwise, if it looks safe to reuse the values of elements across
     multiple coordinates (this will be the case for any inferred strategy, and
     for most of the builtins, but is not the case for mutable values or
